<?php
/**
* Community Builder Fields Searchbot - Extend the standard/advanced Mambo/Joomla search function to include Community Builder Profile/Fields
* @version 1.3
* @creation 23.03.2008
* @author PF
* @authorEmail admin@entw.de
* @package CB Fields Searchbot
* @copyright (C) 2008 www.entw.de
* @license GNU/GPL
* History:
*	27.03.2008 Adding blacklist and title format ability
*	03.04.2008 Display optional Avatar Image and adding various new options for advanced sorting/displaying
*	06.04.2008 Bug fix WHERE clause
*/

// Ensure this file is being included by a parent file
defined( '_VALID_MOS' ) or die( 'Direct Access to this location is not allowed.' );

// Used globals
global $mainframe, $database, $mosConfig_lang, $_MAMBOTS, $Itemid;

// Include CB language file if exists
if ( @file_exists("components/com_comprofiler/plugin/language/$mosConfig_lang/$mosConfig_lang.php") ) {
	require_once("components/com_comprofiler/plugin/language/$mosConfig_lang/$mosConfig_lang.php");
} else if ( @file_exists("components/com_comprofiler/plugin/language/default_language/default_language.php") ) {
	require_once("components/com_comprofiler/plugin/language/default_language/default_language.php");
}
// Include Joomla language file if exists
if ( @file_exists("language/$mosConfig_lang.php") ) {
	require_once("language/$mosConfig_lang.php");
} else if ( @file_exists("language/english.php") ) {
	require_once("language/english.php");
}

// Register the search function inside Joomla/Mambo API
$_MAMBOTS->registerFunction('onSearch', 'botSearchCbFields');

/**
* Search Engine
* @param string $text is the search term
* @param string $matchtype exact, all or any
* @param string $order popular, oldest, newest, alpha or category
* @return array with search results
*/
function botSearchCbFields($text, $matchtype='', $order='') {
	global $database, $mainframe, $mosConfig_lang, $_MAMBOTS, $Itemid;

	if (!trim($text)) return array();
	
	$avatar_image_home = "images/comprofiler";
	$avatar_no_image = "components/com_comprofiler/plugin/language/$mosConfig_lang/images/tnnophoto.jpg";

	// This is the standard blacklist of fields which should never be searched or displayed. Change it if desired.
	// Later in that script this array will be merged with the blacklist array given with the mambot params.
	$stdBlacklist = array('id','text','title','browsernav','password','resultid','href','section','created',
		'sendEmail','gid','registerDate','lastvisitDate','activation','params','fbviewtype','fbordering',
		'fbsignature','acceptedterms','bannedreason','unbannedby','bannedby','unbanneddate','banneddate',
		'banned','cbactivation','registeripaddr','lastupdatedate','confirmed','approved','avatarapproved',
		'avatar','message_number_sent','message_last_sent','usertype');

	// check if param query has previously been processed
	if ( !isset($_MAMBOTS->_search_mambot_params['cb_fields']) ) {
		// load mambot params info
		$query = "SELECT params"
		. "\n FROM #__mambots"
		. "\n WHERE element = 'cb_fields.searchbot'"
		. "\n AND folder = 'search'";
		$database->setQuery($query);
		$database->loadObject($mambot);		
		// save query to class variable
		$_MAMBOTS->_search_mambot_params['cb_fields'] = $mambot;
	}
	// pull query data from class variable
	$mambot = $_MAMBOTS->_search_mambot_params['cb_fields'];	
	$botParams = new mosParameters($mambot->params);

	// Get the bot params
	$Itemid = intval($botParams->def('Itemid', 1));
	$linked_title_format = intval($botParams->def('linked_title_format', 0));
	$section_name = trim($botParams->def('search_result_section_name', 'Profile'));
	$display_size_avatar = $botParams->def('display_size_avatar', 40);
	$display_size_avatar = $display_size_avatar=='' ? intval(-1) : intval($display_size_avatar);
	$limit = intval($botParams->def('search_limit', 50));
	$display_result_fields = trim($botParams->def('display_result_fields', ''));
	$display_empty_fields = intval($botParams->def('display_empty_fields', 1));
	$date_result_field = trim($botParams->def('date_result_field', 'u.registerDate'));
	$advanced_order_field = trim($botParams->def('advanced_order_field', '0'));
	$advanced_order_field = $advanced_order_field=='0' ? $date_result_field : $advanced_order_field;
	$search_fields = trim($botParams->def('search_fields', ''));
	$search_fields = strlen($search_fields)>0 ? array_map('trim',explode(',',$search_fields)) : null;
	$blacklist_search_fields = trim($botParams->def('blacklist_search_fields', ''));
	$blacklist_search_fields = strlen($blacklist_search_fields)>0 ? array_map('trim',explode(',',$blacklist_search_fields)) : array();
	$blacklist_search_fields = array_merge($blacklist_search_fields, $stdBlacklist);

	// Collect all searchable table columns (char, varchar, *text) from table #__comprofiler
	$database->setQuery( 'SHOW FIELDS FROM #__comprofiler');
	$fields = $database->loadObjectList();
	foreach ($fields as $field) { // Ignore all blacklist and non character table columns
		if ( !in_array($field->Field, $blacklist_search_fields) &&
		     (stristr($field->Type,'char')!==false || stristr($field->Type,'text')!==false) ) {
			if ( is_array($search_fields) ) { // Search only given search_fields from searchbot params?
				if ( in_array($field->Field, $search_fields) )
					$colnames[] = "c.".$field->Field;
			} else $colnames[] = "c.".$field->Field; // No, no search_fields given so search all fields
		}
	}
	// CB integer field 'hits' Add here additional int field to display or to be searched (see also Load field titles)
	// Note that they may must be removed from the $stdBlacklist
	$colnames[] = "c.hits";
	// and finally columns from the #__users table
	$colnames[] = "u.username";
	$colnames[] = "u.name";

	// Load field 'titles'/'title defines' from table #__comprofiler_fields
	$database->setQuery("SELECT name, title FROM #__comprofiler_fields");
	$field_titles = $database->loadAssocList('name');
	// CB integer field 'hits' Add here additional col name language translations defines
	$field_titles['hits']['title'] = '_HEADER_HITS';

	// BEGIN WHERE clause depending on matchtype
	$where = '1=1';
	switch ($matchtype) {
		case 'exact': // Exact match of search string in one or more fields
			foreach ($colnames as $colname)
				$where .= "$colname = '$text' \n OR ";
				$where .= '1=0';
			break;
		case 'all': // Each search word must be found at least one time in one or more fields
			$words = explode(' ',$text);
			$where = '(';
			foreach ($words as $word) {
				foreach ($colnames as $colname)
					$where .= "$colname LIKE '%$word%' \n OR ";
				$where = preg_replace("/ OR $/","",$where);
				$where .= ") AND (\n ";
			}
			$where .= '1=1)';
			break;
		case 'any':
		default: // Match one or more of the searched words in one of the fields
			$words = explode(' ',$text);
			$where = "(0=1)";
			foreach ($colnames as $colname) {
				$where .= " OR (0=1";
				foreach ($words as $word)
					$where .= " OR $colname LIKE '%$word%'";
				$where .= ")";
			}
	}
	// Take care to search only for valid profiles
	$where .= "\n AND c.approved=1 AND c.confirmed=1 AND c.banned=0";
	$where .= "\n AND u.name NOT LIKE 'Message %'"; // Ignore a Forum Message Bot if there is one
	// END WHERE

	// BEGIN ORDER BY clause depending on desired order
	switch ($order) {
		case 'popular':
			$order = 'c.hits DESC';
			break;
		case 'oldest':
			$order = "$advanced_order_field ASC";
			break;
		case 'newest':
			$order = "$advanced_order_field DESC";
			break;
		case 'alpha':
		case 'category':
		default:
			$order = 'u.name ASC';
	} 
	// END ORDER BY

	// Build query 'text' column with additional to be displayed result fields from searchbot params
	if ( strlen($display_result_fields) > 0 ) {
		$result = "CONCAT(''"; // Begin CONCAT
		$result_fields = array_map('trim',explode(',',$display_result_fields));
		foreach ($result_fields as $field)
			foreach ($colnames as $col) 
				if ( preg_match("/".preg_quote($field)."$/i",$col) ) {
					$field_title = @$field_titles[$field]['title'];
					// Some field titles in table #__comprofiler_fields are defines so try to get the definition
					if ( defined($field_title) ) eval("\$field_title = $field_title;");
					if ( $display_empty_fields )
						$result .= ",'$field_title: ',IF($col,$col,'".mysql_real_escape_string(_UE_NO_DATA)."'),' '";
					else $result .= ",IF(LENGTH($col)>0,CONCAT('$field_title: ',$col),''),' '";
				}
		$result .= ")"; // End CONCAT
	} else $result = "''";

	// Formating the Avatar if display_size_avatar > -1 was supplied
	if ($display_size_avatar > -1) {
		$display_size_avatar = $display_size_avatar>0 ? "width=\"$display_size_avatar\" " : '';
		$avatar = "IF(LENGTH(c.avatar)>0 AND c.avatarapproved='1',"
		."CONCAT('<img border=\"0\" $display_size_avatar src=\"$avatar_image_home/',c.avatar,'\"\> '),"
		."CONCAT('<img border=\"0\" $display_size_avatar src=\"$avatar_no_image','\"\> ')),";
	} else $avatar = '';

	// Set the format of the linked 'title' column together with the avatar
	if ( $linked_title_format == 1 ) // Name (Username)
		$linked_title_format = "CONCAT($avatar u.name,' (',u.username,')')";
	else if ( $linked_title_format == 2 ) // Firstname Lastname
		$linked_title_format = "CONCAT($avatar c.firstname,' ',c.lastname)";
	else if ( $linked_title_format == 3 ) // Firstname Lastname (Username)
		$linked_title_format = "CONCAT($avatar c.firstname,' ',c.lastname,' (',u.username,')')";
	else $linked_title_format = "CONCAT($avatar '',u.username)"; // 0 default is Username only

	// BEGIN SELECT statement
	$q = "SELECT DISTINCT u.*, c.*, $linked_title_format AS title, \n"
	." $result AS text, \n"
	." $date_result_field AS created, \n"
	." '$section_name' AS section, \n"
	." CONCAT('index.php?option=com_comprofiler&task=userProfile&user=',c.id,'&Itemid=$Itemid') AS href, \n"
	." '2' AS browsernav, \n"
	." u.id AS resultid \n"
	." FROM #__comprofiler c \n"
	." LEFT JOIN #__users u ON u.id = c.user_id \n"
	." WHERE $where \n"
	." ORDER BY $order";
	// END SELECT

	$database->setQuery($q, 0, $limit);
	$results = $database->loadObjectList();
	if ( ! $results ) return array(); // Nothing was found

	// Post SELECT - Find now the search text in all returned colums and append them 
	// to our 'text' column of the result row so that we finally see where all the matches are occured
	$words = array_map('trim',explode(' ',$text));
	for ($i=0; $i < count($results); $i++) {
		foreach (get_object_vars($results[$i]) as $k => $v) {
			// Ignore all unwanted columns, taking into account the blacklist
			// and the search_fields if given via the searchbot params
			if (is_array($v) or is_object($v) or $v === NULL or stristr($k,'password')!=false
			    or in_array($k, $blacklist_search_fields)
			    or (is_array($search_fields) && !in_array($k, $search_fields)) ) {
				continue;
			}
			foreach ($words as $word) { // Report for each word the matched fields
				if ( stristr($v,$word) != false) {
					$field_title = @$field_titles[$k]['title'];
					if ( defined($field_title) ) eval("\$field_title = $field_title;");
					// but ignore dublicate matched fields
					if ( strstr($results[$i]->text, "$field_title: $v ") == false)
						$results[$i]->text .= "$field_title: $v ";
				}
			}
		}
	}

	return $results;
} // End botSearchCbFields
?>
